
local self = {}

local img = love.image.newImageData(32,32)
local dif = function(x,y)
	return math.max(x,y) - math.min(x,y)
end
local sqrdis = function(x1,y1,x2,y2)
	return math.max(math.max(x1,x2) - math.min(x1,x2),math.max(y1,y2) - math.min(y1,y2))
end
for i = 1,32 do
	for j = 1,32 do
		local d = 255 - math.max(0,sqrdis(i-0.5,j-0.5,16,16) / 16 - 0.7) * 255
		img:setPixel(i-1,j-1,d,d,d,d)
	end
end
sqrdis = nil
dif = nil
img = love.graphics.newImage(img)
local smallfont = love.graphics.newFont( "PressStart2P-Regular.ttf", 8)
local mediumfont = love.graphics.newFont( "PressStart2P-Regular.ttf", 32)
local bigfont = love.graphics.newFont( "PressStart2P-Regular.ttf", 64)

function self.new()
	local self = {}
	local ended = false
	local board = {}
	for i = 1,8 do
		board[i] = {}
		for j = 1,8 do
			if i >= 4 and i <= 5 and j >= 4 and j <= 5 then
				if i == j then
					board[i][j] = 2
				else
					board[i][j] = 1
				end
			else
				board[i][j] = 0
			end
		end
	end
	local turn = 1
	local player = {
		{c={r=255,g=255,b=255},s=2},
		{c={r=0,g=0,b=0},s=2},
	}
	local function trace(x,y)
		local trace = {
			[-1]={[-1]=1,[0]=1,[1]=1},
			[0]={[-1]=1,[0]=0,[1]=1},
			[1]={[-1]=1,[0]=1,[1]=1},
		}
		for i = 1,9 do
			for j = -1,1 do
				for k = -1,1 do
					if not (j == 0 and k == 0) and trace[j][k] == i then
						if x + trace[j][k] * j >= 1 and x + trace[j][k] * j <= 8 and y + trace[j][k] * k >= 1 and y + trace[j][k] * k <= 8 then
							if board[x + trace[j][k] * j][y + trace[j][k] * k] ~= 0 and board[x + trace[j][k] * j][y + trace[j][k] * k] ~= math.mod(turn-1,2)+1 then
								trace[j][k] = i + 1
							end
						end
					end
				end
			end
		end
		return trace
	end
	local function score_update()
		local free = false
		local move = false
		player[1].s = 0
		player[1].hd = false
		player[2].s = 0
		player[2].hd = false
		for i = 1,8 do
			for j = 1,8 do
				if board[i][j] == math.mod(turn-1,2)+1 then
					local trace = trace(i,j)
					for k = -1,1 do
						for l = -1,1 do
							if not (k == 0 and l == 0) and trace[k][l] ~= 1 then
								if board[i + trace[k][l] * k] ~= nil then
									if board[i + trace[k][l] * k][j + trace[k][l] * l] == 0 then
										move = true
									end
								end
							end
						end
					end
				end
				if board[i][j] ~= 0 then
					player[board[i][j]].s = player[board[i][j]].s + 1
					player[board[i][j]].hd = true
				else
					free = true
				end
			end
		end
		print(move)
		if not move or not free or not player[1].hd or not player[2].hd then
			ended = true
		end
	end
	function self:click(mx,my)
		local cx,cy = math.floor(mx/60),math.floor(my/60)
		if cx >= 1 and cx <= 8 and cy >= 1 and cy <= 8 and board[cx][cy] == 0 then
			local trace = trace(cx,cy)
			local taken = false
			for i = -1,1 do
				for j = -1,1 do
					if cx + trace[i][j] * i >= 1 and cx + trace[i][j] * i <= 8 and cy + trace[i][j] * j >= 1 and cy + trace[i][j] * j <= 8 then
						if board[cx + trace[i][j] * i][cy + trace[i][j] * j] == math.mod(turn-1,2)+1 and not (i == 0 and j == 0) then
							for k = 1,trace[i][j] - 1 do
								board[cx + k * i][cy + k * j] = math.mod(turn-1,2)+1
								taken = true
							end
						end
					end
				end
			end
			if taken then
				board[cx][cy] = math.mod(turn-1,2)+1
				turn = turn + 1
				score_update()
			end
		end
	end
	function self:update(dt)
		
	end
	function self:draw()
		love.graphics.setColor(0,65,0)
		love.graphics.rectangle("fill",0,0,800,600)
		local turn = math.mod(turn-1,2)+1
		for i = 1,8 do
			for j = 1,8 do
				if board[i][j] ~= 0 then
					love.graphics.setColor(player[board[i][j]].c.r,player[board[i][j]].c.g,player[board[i][j]].c.b)
					love.graphics.rectangle("fill",i * 60 + 10,j * 60 + 10,40,40)
				end
			end
		end
		love.graphics.setColor(0,0,0)
		for i = 1,9 do
			love.graphics.line(i * 60,60,i * 60,540)
			love.graphics.line(60,i * 60,540,i * 60)
		end
		local mx,my = love.mouse.getX(),love.mouse.getY()
		if mx > 60 and mx < 540 and my > 60 and my < 540 then
			local xx,yy
			xx = math.floor(mx / 60)
			yy = math.floor(my / 60)
			if board[xx][yy] == 0 then
				love.graphics.setColor(player[turn].c.r,player[turn].c.g,player[turn].c.b)
				love.graphics.rectangle("fill",xx * 60 + 10,yy * 60 + 10,40,40)
			end
		end
		for i = 1, #player do
			love.graphics.setFont(bigfont)
			love.graphics.setColor(player[i].c.r,player[i].c.g,player[i].c.b)
			love.graphics.print(player[i].s,700,i * 300 - 150,nil,nil,nil,bigfont:getWidth(player[i].s) / 2,bigfont:getHeight() / 2)
			if ended then
				love.graphics.setColor(0,65,0)
				love.graphics.rectangle("fill",250,150,300,300)
				love.graphics.setColor(0,0,0)
				love.graphics.rectangle("line",250,150,300,300)
				love.graphics.print("GAME",400,300,nil,nil,nil,bigfont:getWidth("GAME") / 2,bigfont:getHeight())
				love.graphics.print("OVER",400,300,nil,nil,nil,bigfont:getWidth("OVER") / 2,nil)
			end
		end
	end
	return self
end

return self
